@@ -155,7 +155,6 @@ static CURLcode do_sendmsg(struct Curl_cfilter *cf,
   }
 #endif
 
-
   while((sent = sendmsg(qctx->sockfd, &msg, 0)) == -1 &&
         SOCKERRNO == SOCKEINTR)
     ;
@@ -254,7 +253,7 @@ out:
   CURL_TRC_CF(data, cf, "vquic_%s(len=%zu, gso=%zu, calls=%zu)"
               " -> %d, sent=%zu",
               VQUIC_SEND_METHOD, pktlen, gsolen, calls, result, *psent);
-  return CURLE_OK;
+  return result;
 }
 
 static CURLcode vquic_send_packets(struct Curl_cfilter *cf,
@@ -378,9 +377,16 @@ static CURLcode recvmmsg_packets(struct Curl_cfilter *cf,
                                  struct Curl_easy *data,
                                  struct cf_quic_ctx *qctx,
                                  size_t max_pkts,
-                                 vquic_recv_pkt_cb *recv_cb, void *userp)
+                                 vquic_recv_pkts_cb *recv_cb, void *userp)
 {
+#if defined(__linux__) && defined(UDP_GRO)
 #define MMSG_NUM  16
+#define UDP_GRO_CNT_MAX  64
+#else
+#define MMSG_NUM  64
+#define UDP_GRO_CNT_MAX  1
+#endif
+#define MSG_BUF_SIZE  (UDP_GRO_CNT_MAX * 1500)
   struct iovec msg_iov[MMSG_NUM];
   struct mmsghdr mmsg[MMSG_NUM];
   uint8_t msg_ctrl[MMSG_NUM * CMSG_SPACE(sizeof(int))];
@@ -390,17 +396,15 @@ static CURLcode recvmmsg_packets(struct Curl_cfilter *cf,
   char errstr[STRERROR_LEN];
   CURLcode result = CURLE_OK;
   size_t gso_size;
-  size_t pktlen;
-  size_t offset, to;
   char *sockbuf = NULL;
-  uint8_t (*bufs)[64*1024] = NULL;
+  uint8_t (*bufs)[MSG_BUF_SIZE] = NULL;
 
   DEBUGASSERT(max_pkts > 0);
-  result = Curl_multi_xfer_sockbuf_borrow(data, MMSG_NUM * sizeof(bufs[0]),
+  result = Curl_multi_xfer_sockbuf_borrow(data, MMSG_NUM * MSG_BUF_SIZE,
                                           &sockbuf);
   if(result)
     goto out;
-  bufs = (uint8_t (*)[64*1024])sockbuf;
+  bufs = (uint8_t (*)[MSG_BUF_SIZE])sockbuf;
 
   total_nread = 0;
   while(pkts < max_pkts) {
@@ -449,22 +453,12 @@ static CURLcode recvmmsg_packets(struct Curl_cfilter *cf,
         gso_size = mmsg[i].msg_len;
       }
 
-      for(offset = 0; offset < mmsg[i].msg_len; offset = to) {
-        ++pkts;
-
-        to = offset + gso_size;
-        if(to > mmsg[i].msg_len) {
-          pktlen = mmsg[i].msg_len - offset;
-        }
-        else {
-          pktlen = gso_size;
-        }
-
-        result = recv_cb(bufs[i] + offset, pktlen, mmsg[i].msg_hdr.msg_name,
-                         mmsg[i].msg_hdr.msg_namelen, 0, userp);
-        if(result)
-          goto out;
-      }
+      result = recv_cb(bufs[i], mmsg[i].msg_len, gso_size,
+                       mmsg[i].msg_hdr.msg_name,
+                       mmsg[i].msg_hdr.msg_namelen, 0, userp);
+      if(result)
+        goto out;
+      pkts += (mmsg[i].msg_len + gso_size - 1) / gso_size;
     }
   }
 
@@ -481,7 +475,7 @@ static CURLcode recvmsg_packets(struct Curl_cfilter *cf,
                                 struct Curl_easy *data,
                                 struct cf_quic_ctx *qctx,
                                 size_t max_pkts,
-                                vquic_recv_pkt_cb *recv_cb, void *userp)
+                                vquic_recv_pkts_cb *recv_cb, void *userp)
 {
   struct iovec msg_iov;
   struct msghdr msg;
@@ -494,8 +488,6 @@ static CURLcode recvmsg_packets(struct Curl_cfilter *cf,
   CURLcode result = CURLE_OK;
   uint8_t msg_ctrl[CMSG_SPACE(sizeof(int))];
   size_t gso_size;
-  size_t pktlen;
-  size_t offset, to;
 
   DEBUGASSERT(max_pkts > 0);
   for(pkts = 0, total_nread = 0, calls = 0; pkts < max_pkts;) {
@@ -542,20 +534,11 @@ static CURLcode recvmsg_packets(struct Curl_cfilter *cf,
       gso_size = nread;
     }
 
-    for(offset = 0; offset < nread; offset = to) {
-      ++pkts;
-
-      to = offset + gso_size;
-      if(to > nread)
-        pktlen = nread - offset;
-      else
-        pktlen = gso_size;
-
-      result =
-        recv_cb(buf + offset, pktlen, msg.msg_name, msg.msg_namelen, 0, userp);
-      if(result)
-        goto out;
-    }
+    result = recv_cb(buf, nread, gso_size,
+                     msg.msg_name, msg.msg_namelen, 0, userp);
+    if(result)
+      goto out;
+    pkts += (nread + gso_size - 1) / gso_size;
   }
 
 out:
@@ -570,7 +553,7 @@ static CURLcode recvfrom_packets(struct Curl_cfilter *cf,
                                  struct Curl_easy *data,
                                  struct cf_quic_ctx *qctx,
                                  size_t max_pkts,
-                                 vquic_recv_pkt_cb *recv_cb, void *userp)
+                                 vquic_recv_pkts_cb *recv_cb, void *userp)
 {
   uint8_t buf[64*1024];
   int bufsize = (int)sizeof(buf);
@@ -611,8 +594,8 @@ static CURLcode recvfrom_packets(struct Curl_cfilter *cf,
     ++pkts;
     ++calls;
     total_nread += (size_t)nread;
-    result = recv_cb(buf, (size_t)nread, &remote_addr, remote_addrlen,
-                     0, userp);
+    result = recv_cb(buf, (size_t)nread, (size_t)nread,
+                     &remote_addr, remote_addrlen, 0, userp);
     if(result)
       goto out;
   }
@@ -629,7 +612,7 @@ CURLcode vquic_recv_packets(struct Curl_cfilter *cf,
                             struct Curl_easy *data,
                             struct cf_quic_ctx *qctx,
                             size_t max_pkts,
-                            vquic_recv_pkt_cb *recv_cb, void *userp)
+                            vquic_recv_pkts_cb *recv_cb, void *userp)
 {
   CURLcode result;
 #ifdef HAVE_SENDMMSG
